#include <18F2550.h>
#device adc=10
#fuses HSPLL,NOWDT,NOMCLR,NOPROTECT,NOLVP,NODEBUG,USBDIV,PLL5,CPUDIV1,VREGEN
#use delay(clock=48000000)

#define use_portb_lcd TRUE

#include <usb_cdc.h>
#include <Flex_LCD420.h>
#include <MATH.H>

float U,I,T,A;

void InitDisplay()
{
   lcd_gotoxy(1, 4);
   printf (lcd_putc,"!1!2!3!4!5!6!7!8!9!#");
}

void DisplayValues()
{
   float P, R;
   
   P = U*I;
   if (I>0.01)  R = U/I;
   else R=100;
   
   lcd_gotoxy(1, 1);
   printf (lcd_putc,"U:%02.1fV ",U);
   lcd_gotoxy(9, 1);
   printf (lcd_putc,"I:%1.2fA ",I);
   lcd_gotoxy(18, 1);
   printf (lcd_putc,"Tmp");
   lcd_gotoxy(1, 2);
   printf (lcd_putc,"P:%03.0fW ",P);
   lcd_gotoxy(9, 2);
   if (R<100) 
   {
              printf (lcd_putc,"R:%02.1f",R);
              lcd_putc(0xF4);
              lcd_putc(0xFE);
   }           
   else       printf (lcd_putc,"R:Over! ");
   lcd_gotoxy(17, 2);
   printf (lcd_putc,"%2.1f",T);
}

void BarGraph()
{
   int Value,j;
   
   Value = (int)(I*2);
   
   for (j=1;j<21;j++)
   {
    lcd_gotoxy(j, 3);
    if (Value>=j) lcd_putc (0xFF);
    else          lcd_putc (0xFE);
   }
}

#int_TIMER1 
void  TIMER1_isr(void) 
{
    static int qmillisec = 0;
     
   set_timer1(5536);
    
   if (qmillisec++==25) //Action toute les secondes
   {
      DisplayValues();
      qmillisec=0;
   }
   if ((qmillisec%5)==0) 
   {
      BarGraph();
   }   
}

void main()
{
   float Voltage,LogRes;
   char c;
   
   setup_adc_ports(AN0_TO_AN3|VSS_VDD);
   setup_adc(ADC_CLOCK_INTERNAL);
   setup_vref(FALSE);
   
   setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);
   enable_interrupts(INT_TIMER1);
   enable_interrupts(GLOBAL);
   usb_cdc_init();
   usb_init();
   lcd_init();
   delay_ms(10);
   InitDisplay();
   
   output_high(PIN_C0); //Activation de l'alimentation
   
   while (TRUE) 
   {
      //Lecture des Tensions
      set_adc_channel(0);
      delay_us(50);
      U = (float)(read_adc())*35.0/1023;
      set_adc_channel(1);
      delay_us(50);
      I = (float)(read_adc())*10.0/1023;
      set_adc_channel(2);
      delay_us(50);
      Voltage = (float)(read_adc());
      if (Voltage > 1020) Voltage = 1020;
      LogRes = log(10.0*Voltage/(1023-Voltage));
      T = (1/(0.00277071+0.00025184*LogRes+0.00000028598*LogRes*LogRes*LogRes)-273.16);
      set_adc_channel(3);
      delay_us(50);
      A = (float)(read_adc())*5.0/1023;
      
      //Sorties Relais
      if (T>60.0) output_low(PIN_C0);
      if (T<55.0) output_high(PIN_C0);
      
      if (U>19.0) output_high(PIN_C1);
      if (U<17.0) output_low(PIN_C1);
      
      //Gestion USB
      usb_task();
      enable_interrupts(GLOBAL);
      if (usb_cdc_kbhit()) 
      {
         c=usb_cdc_getc();
         if (c=='U') printf(usb_cdc_putc, "%f\r\n",U);
         if (c=='I') printf(usb_cdc_putc, "%f\r\n",I);
         if (c=='T') printf(usb_cdc_putc, "%f\r\n",T);
         if (c=='A') printf(usb_cdc_putc, "%f\r\n",A);
         if (c=='?') printf(usb_cdc_putc, "Test Ok\r\n");
         enable_interrupts(GLOBAL);
      }   
                 
   }
}



